<template>
    <page-container>
        <div class="device-list">
            <div class="header-title-container">
                <div class="header-title-text">设备列表</div>
            </div>
            <div class="device-list-content">
                <div class="device-list-item">
                    <div class="device-list-item-content">
                        <img
                            style="width: 70px; height: 70px;"
                            :src="getImage('/device-manager/count_icon1.png')"
                            alt=""
                        />
                        <div>
                            <div class="count-text">285</div>
                            <div class="count-label">待处理告警</div>
                        </div>
                    </div>
                </div>
                <div
                    v-for="(item, index) in devices"
                    :key="index"
                    class="device-list-item"
                >
                    <div class="device-list-item-title">仓库1</div>
                    <div class="device-list-item-content">
                        <img :src="getImage(item.imgUrl)" alt="" />
                        <div>
                            <div class="count-text">{{ item.count }}</div>
                            <div class="count-label">{{ item.countLabel }}</div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
        <pro-search
            :columns="columns"
            target="device-instance"
            @search="handleSearch"
        />
        <FullPage>
            <JProTable
                ref="instanceRef"
                :columns="columns"
                :request="query"
                :defaultParams="{
                    sorts: [{ name: 'createTime', order: 'desc' }],
                }"
                model="TABLE"
                :rowSelection="
                    isCheck
                        ? {
                              selectedRowKeys: _selectedRowKeys,
                              onChange: onSelectChange,
                          }
                        : false
                "
                :params="params"
            >
                <template #headerTitle>
                    <j-space>
                        <PermissionButton
                            type="primary"
                            @click="handleAdd"
                            hasPermission="device/Instance:add"
                        >
                            <template #icon>
                                <AIcon type="PlusOutlined" />
                            </template>
                            新增
                        </PermissionButton>
                    </j-space>
                </template>
                <template #state="slotProps">
                    <BadgeStatus
                        :status="slotProps.state?.value"
                        :text="slotProps.state?.text"
                        :statusNames="{
                            online: 'processing',
                            offline: 'error',
                            notActive: 'warning',
                        }"
                    />
                </template>
                <template #createTime="slotProps">
                    <span>{{
                        slotProps?.createTime
                            ? dayjs(slotProps.createTime).format(
                                  'YYYY-MM-DD HH:mm:ss',
                              )
                            : ''
                    }}</span>
                </template>
                <template #action="slotProps">
                    <j-space>
                        <template
                            v-for="i in getActions(slotProps, 'table')"
                            :key="i.key"
                        >
                            <PermissionButton
                                :disabled="i.disabled"
                                :popConfirm="i.popConfirm"
                                :tooltip="{
                                    ...i.tooltip,
                                }"
                                @click="i.onClick"
                                type="link"
                                style="padding: 0 5px"
                                :danger="i.key === 'delete'"
                                :hasPermission="
                                    i.key === 'view'
                                        ? true
                                        : 'device/Instance:' + i.key
                                "
                            >
                                <template #icon>
                                    <AIcon :type="i.icon" />
                                </template>
                            </PermissionButton>
                        </template>
                    </j-space>
                </template>
            </JProTable>
        </FullPage>
    </page-container>
</template>

<script setup lang="ts">
import {
    query,
    _delete,
    _deploy,
    _undeploy,
} from '@/api/device/instance';
import { getImage, LocalStore } from '@/utils/comm';
import { message } from 'jetlinks-ui-components';
import { useMenuStore } from '@/store/menu';
import type { ActionsType } from './typings';
import dayjs from 'dayjs';
import BadgeStatus from '@/components/BadgeStatus/index.vue';
import { BatchActionsType } from '@/components/BatchDropdown/types';
import { useRouterParams } from '@/utils/hooks/useParams';

const instanceRef = ref<Record<string, any>>({});
const params = ref<Record<string, any>>({});
const _selectedRowKeys = ref<string[]>([]);
const importVisible = ref<boolean>(false);
const exportVisible = ref<boolean>(false);
const visible = ref<boolean>(false);
const current = ref<Record<string, any>>({});
const operationVisible = ref<boolean>(false);
const api = ref<string>('');
const type = ref<string>('');
const isCheck = ref<boolean>(true);
const routerParams = useRouterParams();
const menuStory = useMenuStore();
const devices = ref([
    {
        imgUrl: '/device-manager/count_icon1.png',
        count: 64,
        countLabel: '设备总量',
        toolbars: [
            { name: '当前在线', value: 32 },
            { name: '离线设备', value: 32 },
        ],
    },
    {
        imgUrl: '/device-manager/count_icon2.png',
        count: 64,
        countLabel: 'XX库房',
        toolbars: [
            { name: '当前在线', value: 32 },
            { name: '离线设备', value: 32 },
            { name: '产品数量', value: 32 },
        ],
    },
]);

const transformData = (arr: any[]): any[] => {
    if (Array.isArray(arr) && arr.length) {
        return (arr || []).map((item: any) => {
            return {
                ...item,
                id: `classifiedId is ${item.id}`,
                children: transformData(item.children),
            };
        });
    } else {
        return [];
    }
};

const columns = [
    {
        title: 'ID',
        dataIndex: 'id',
        key: 'id',
        ellipsis: true,
        search: {
            type: 'string',
            defaultTermType: 'eq',
        },
    },
    {
        title: '告警编号',
        dataIndex: 'number',
        key: 'number',
        ellipsis: true,
        search: {
            type: 'string',
        },
    },
    {
        title: '设备名称',
        dataIndex: 'name',
        key: 'name',
        ellipsis: true,
        search: {
            type: 'string',
            first: true,
        },
    },
    {
        title: '发生位置',
        dataIndex: 'position',
        key: 'position',
        ellipsis: true,
        search: {
            type: 'string',
        },
    },
    {
        title: '告警类型',
        dataIndex: 'position',
        key: 'position',
        ellipsis: true,
        search: {
            type: 'string',
        },
    },
    {
        title: '告警描述',
        dataIndex: 'describe',
        key: 'describe',
        ellipsis: true,
        search: {
            type: 'string',
        },
    },
    {
        title: '值',
        dataIndex: 'value',
        key: 'value',
        ellipsis: true,
        search: {
            type: 'string',
        },
    },
    {
        title: '告警时间',
        dataIndex: 'type',
        key: 'type',
        ellipsis: true,
        search: {
            type: 'string',
        },
    },
    {
        title: '操作',
        key: 'action',
        fixed: 'right',
        width: 200,
        scopedSlots: true,
    },
];

const paramsFormat = (
    config: Record<string, any>,
    _terms: Record<string, any>,
    name?: string,
) => {
    if (
        config?.terms &&
        Array.isArray(config.terms) &&
        config?.terms.length > 0
    ) {
        (config?.terms || []).map(
            (item: Record<string, any>, index: number) => {
                if (item?.type) {
                    _terms[`${name ? `${name}.` : ''}terms[${index}].type`] =
                        item.type;
                }
                paramsFormat(
                    item,
                    _terms,
                    `${name ? `${name}.` : ''}terms[${index}]`,
                );
            },
        );
    } else if (!config?.terms && Object.keys(config).length > 0) {
        Object.keys(config).forEach((key) => {
            if (config[key]) {
                _terms[`${name ? `${name}.` : ''}${key}`] = config[key];
            }
        });
    }
};

onMounted(() => {
    if (routerParams.params.value.type === 'add') {
        handleAdd();
    }
    if (routerParams.params.value.type === 'import') {
        importVisible.value = true;
    }
});

const handleParams = (config: Record<string, any>) => {
    const _terms: Record<string, any> = {};
    paramsFormat(config, _terms);
    if (Object.keys(_terms).length) {
        const url = new URLSearchParams();
        Object.keys(_terms).forEach((key) => {
            console.log(_terms[key]);
            url.append(key, _terms[key]);
        });
        return url.toString();
    } else {
        return '';
    }
};

/**
 * 新增
 */
const handleAdd = () => {
    visible.value = true;
    current.value = {};
};

/**
 * 查看
 */
const handleView = (id: string) => {
    menuStory.jumpPage('device-warning/Detail', { id });
};

const getActions = (
    data: Partial<Record<string, any>>,
    type: 'card' | 'table',
): ActionsType[] => {
    if (!data) return [];
    const actions = [
        {
            key: 'view',
            text: '详情',
            tooltip: {
                title: '详情',
            },
            icon: 'FileTextOutlined',
            onClick: () => {
                handleView(data.id);
            },
        },
        {
            key: 'update',
            text: '处理',
            tooltip: {
                title: '处理',
            },
            icon: 'EditOutlined',
            onClick: () => {
                visible.value = true;
                current.value = data;
            },
        },
        {
            key: 'delete',
            text: '忽略',
            disabled: data.state?.value !== 'notActive',
            tooltip: {
                title:
                    data.state.value !== 'notActive'
                        ? '已启用的设备不能删除'
                        : '删除',
            },
            popConfirm: {
                title: '确认删除?',
                onConfirm: async () => {
                    const resp = await _delete(data.id);
                    if (resp.status === 200) {
                        message.success('操作成功！');
                        instanceRef.value?.reload();
                    } else {
                        message.error('操作失败！');
                    }
                },
            },
            icon: 'DeleteOutlined',
        },
    ];
    // if (type === 'card')
    //     return actions.filter((i: ActionsType) => i.key !== 'view');
    return actions;
};

const onSelectChange = (keys: string[]) => {
    _selectedRowKeys.value = [...keys];
};

const handleSearch = (_params: any) => {
    // params.value = _params;
    const newParams = (_params?.terms as any[])?.map((item1) => {
        item1.terms = item1.terms.map((item2: any) => {
            if (
                item2.column &&
                ['classifiedId', 'accessId', 'accessProvider'].includes(
                    item2.column,
                )
            ) {
                return {
                    ...item2,
                    column: 'productId$product-info',
                };
            }
            return item2;
        });
        return item1;
    });
    params.value = { terms: newParams || [] };
};
</script>

<style lang="less">
.device-list {
    background: white;
    margin-bottom: 24px;
    .device-list-content {
        display: flex;
        padding: 16px;
        .device-list-item + .device-list-item {
            margin-left: 16px;
        }
        .device-list-item {
            width: 338px;
            height: 105px;
            background: #ffffff;
            box-shadow: 0px 0px 10px 0px rgba(0, 0, 0, 0.15);
            border-radius: 8px 8px 8px 8px;

            .device-list-item-title {
                font-size: 16px;
                font-family: PingFang SC-Medium, PingFang SC;
                font-weight: 500;
                color: #4f4f4f;
                line-height: 23px;
                padding: 3px 16px;
                border-bottom: 1px solid #ececec;
            }
            .device-list-item-content {
                display: flex;
                align-items: center;
                padding: 14px 28px;
                img {
                    width: 48px;
                    height: 48px;
                    margin-right: 19px;
                }
                .count-text {
                    font-size: 28px;
                    font-family: DIN Alternate-Bold, DIN Alternate;
                    font-weight: bold;
                    color: #333333;
                    line-height: 36px;
                    // margin-bottom: 8px;
                }
                .count-label {
                    font-size: 14px;
                    font-family: PingFang SC-Regular, PingFang SC;
                    font-weight: 400;
                    color: #828282;
                    line-height: 16px;
                }
            }
        }
    }
}
.header-title-container {
    padding: 16px 0 16px 24px;
    border-bottom: 1px solid #e6e6e6;
    .header-title-text {
        border-left: 3px solid #4277ff;
        padding-left: 5px;
        font-size: 16px;
        font-family: PingFang SC-Medium, PingFang SC;
        font-weight: 500;
        color: #000000;
        line-height: 24px;
    }
}
.card-button {
    .card-action {
        flex: 1;
        text-align: center;
        color: #3468ff;
    }
}

.card-button.delete {
    .card-action {
        color: #fa3e2e;
    }
}
</style>
